package com.semarchy.mdm.client.samples;

import java.util.Locale;
import java.util.UUID;

import com.semarchy.mdm.client.IDataEditionConnection;
import com.semarchy.mdm.client.IMDMConnection;
import com.semarchy.mdm.client.MDMConnectionFactory;
import com.semarchy.mdm.runtime.activity.ActivityPriority;
import com.semarchy.mdm.runtime.activity.CompletionOptions;
import com.semarchy.mdm.runtime.activity.IActivityService;
import com.semarchy.mdm.runtime.activity.IActivityService.ITaskCompletionResult;
import com.semarchy.mdm.runtime.activity.IActivityTask;
import com.semarchy.mdm.runtime.data.DataQuery;
import com.semarchy.mdm.runtime.data.IDataAccessService;
import com.semarchy.mdm.runtime.data.IEntityRow;
import com.semarchy.mdm.runtime.model.dataaccess.AttributeNames;
import com.semarchy.mdm.runtime.model.logical.IApplication;
import com.semarchy.mdm.runtime.model.logical.IDataEntryWorkflow;
import com.semarchy.mdm.runtime.model.logical.IEntity;
import com.semarchy.mdm.runtime.model.logical.IModel;

public class Sample6_CreateContactWorkflow {

	public static void main(String[] args) {
		IMDMConnection conn = MDMConnectionFactory.connect(
				SamplesConstants.SEMARCHY_URL, SamplesConstants.USERNAME,
				SamplesConstants.PASSWORD);

		IDataEditionConnection dataEditionConnection = conn
				.createDataEditionConnection(SamplesConstants.GS_DATAEDITION);

		IModel logicalModel = dataEditionConnection.getLogicalModel();
		IApplication demoApplication = logicalModel.getApplication(SamplesConstants.GS_DEMO_APP);
		
		
		IDataEntryWorkflow createContactWorkflow = demoApplication.getDataEntryWorkflow("ContactsCreationProcess");
		
		IActivityService activityService = dataEditionConnection
				.getActivityService();
		IDataAccessService dataService = dataEditionConnection
				.getDataAccessService();

		// Start a DataEntry Activity
		IActivityTask createContactTask = activityService.startActivity(
				createContactWorkflow.getTask("CreateOrModifyContacts"),
				"Client API Samples (Sample6_CreateContactWorkflow)",
				null,
				ActivityPriority.HIGH, true, false, null);
		IEntity contactEntity = logicalModel.getEntity("Contact");
		
		
		IEntityRow newContact = dataService.newRow(createContactTask, contactEntity);
		newContact.setValue(AttributeNames.sourceIdName(newContact.getView()), UUID.randomUUID().toString()); 
		newContact.setValue("FirstName", "Jim");
		newContact.setValue("LastName", "Kilker");
		newContact.setValue("Gender", "M");
		newContact.setValue("Phone1", "555");
		
		DataQuery qryCustomer = new DataQuery(dataEditionConnection.getDataAccessModel().getDataEntryLookupView(logicalModel.getEntity("Customer")), createContactTask.getLoadID());
		qryCustomer.setWhereCondition("CustomerName = 'OSATO CHEMICALS'");
		
		newContact.setReference("Customer", dataService.queryFirstEntityRow(qryCustomer));
		dataService.persistRow(createContactTask.getTaskUUID(), newContact);

		// Complete Task and submit workflow 
		ITaskCompletionResult completeCreateContactTask = activityService.completeTask(createContactTask.getTaskUUID(), 
				"SendtoVerification",								
				"Please review contact info", CompletionOptions.claimNextTask());
		
	
		if (completeCreateContactTask.isCompleted()) {
			System.out.println("Task Completed new task is "
					+ completeCreateContactTask.getNewTask());	
		} else {
			System.err.println("Task not completed transition failed: "
					+ completeCreateContactTask.getTransitionValidationReport().getSummary(Locale.getDefault()));
			System.exit(1);
		}
		
		IActivityTask verifyTask = completeCreateContactTask.getNewTask();
		
		
		// Submit
		ITaskCompletionResult submitResult = activityService.completeTask(verifyTask.getTaskUUID(),
				verifyTask.getHumanTask().getSubmitTransition(),								
				"Every thing is ok submitting", null);
		
		if (submitResult.isCompleted()) {
			System.out.println("Contact Creation submitted in batch number "
					+ submitResult.getSubmittedBatchID());	
		} else {
			System.err.println("Task not completed transition failed: "
					+ submitResult.getTransitionValidationReport().getSummary(Locale.getDefault()));
			System.exit(1);
		}
		
		
	}
}
